@@ -30,15 +30,39 @@ std::vector<std::pair<std::string, std::string>> algorithm_specs_for_group(Group
          return {{"Curve25519", "Curve25519"}, {"Kyber", "Kyber-512-r3"}};
       case Group_Params::HYBRID_X25519_KYBER_768_R3_OQS:
          return {{"Curve25519", "Curve25519"}, {"Kyber", "Kyber-768-r3"}};
+      case Group_Params::HYBRID_X448_KYBER_768_R3_OQS:
+         return {{"X448", "X448"}, {"Kyber", "Kyber-768-r3"}};
+      case Group_Params::HYBRID_X25519_eFRODOKEM_640_SHAKE_OQS:
+         return {{"Curve25519", "Curve25519"}, {"FrodoKEM", "eFrodoKEM-640-SHAKE"}};
+      case Group_Params::HYBRID_X25519_eFRODOKEM_640_AES_OQS:
+         return {{"Curve25519", "Curve25519"}, {"FrodoKEM", "eFrodoKEM-640-AES"}};
+      case Group_Params::HYBRID_X448_eFRODOKEM_976_SHAKE_OQS:
+         return {{"X448", "X448"}, {"FrodoKEM", "eFrodoKEM-976-SHAKE"}};
+      case Group_Params::HYBRID_X448_eFRODOKEM_976_AES_OQS:
+         return {{"X448", "X448"}, {"FrodoKEM", "eFrodoKEM-976-AES"}};
 
       case Group_Params::HYBRID_SECP256R1_KYBER_512_R3_OQS:
          return {{"ECDH", "secp256r1"}, {"Kyber", "Kyber-512-r3"}};
       case Group_Params::HYBRID_SECP256R1_KYBER_768_R3_OQS:
          return {{"ECDH", "secp256r1"}, {"Kyber", "Kyber-768-r3"}};
+      case Group_Params::HYBRID_SECP256R1_eFRODOKEM_640_SHAKE_OQS:
+         return {{"ECDH", "secp256r1"}, {"FrodoKEM", "eFrodoKEM-640-SHAKE"}};
+      case Group_Params::HYBRID_SECP256R1_eFRODOKEM_640_AES_OQS:
+         return {{"ECDH", "secp256r1"}, {"FrodoKEM", "eFrodoKEM-640-AES"}};
+
       case Group_Params::HYBRID_SECP384R1_KYBER_768_R3_OQS:
          return {{"ECDH", "secp384r1"}, {"Kyber", "Kyber-768-r3"}};
+      case Group_Params::HYBRID_SECP384R1_eFRODOKEM_976_SHAKE_OQS:
+         return {{"ECDH", "secp384r1"}, {"FrodoKEM", "eFrodoKEM-976-SHAKE"}};
+      case Group_Params::HYBRID_SECP384R1_eFRODOKEM_976_AES_OQS:
+         return {{"ECDH", "secp384r1"}, {"FrodoKEM", "eFrodoKEM-976-AES"}};
+
       case Group_Params::HYBRID_SECP521R1_KYBER_1024_R3_OQS:
          return {{"ECDH", "secp521r1"}, {"Kyber", "Kyber-1024-r3"}};
+      case Group_Params::HYBRID_SECP521R1_eFRODOKEM_1344_SHAKE_OQS:
+         return {{"ECDH", "secp521r1"}, {"FrodoKEM", "eFrodoKEM-1344-SHAKE"}};
+      case Group_Params::HYBRID_SECP521R1_eFRODOKEM_1344_AES_OQS:
+         return {{"ECDH", "secp521r1"}, {"FrodoKEM", "eFrodoKEM-1344-AES"}};
 
       default:
          return {};
@@ -79,13 +103,37 @@ std::vector<size_t> public_value_lengths_for_group(Group_Params group) {
          return {32, 800};
       case Group_Params::HYBRID_X25519_KYBER_768_R3_OQS:
          return {32, 1184};
+      case Group_Params::HYBRID_X448_KYBER_768_R3_OQS:
+         return {56, 1184};
+      case Group_Params::HYBRID_X25519_eFRODOKEM_640_SHAKE_OQS:
+         return {32, 9616};
+      case Group_Params::HYBRID_X25519_eFRODOKEM_640_AES_OQS:
+         return {32, 9616};
+      case Group_Params::HYBRID_X448_eFRODOKEM_976_SHAKE_OQS:
+         return {56, 15632};
+      case Group_Params::HYBRID_X448_eFRODOKEM_976_AES_OQS:
+         return {56, 15632};
 
       case Group_Params::HYBRID_SECP256R1_KYBER_512_R3_OQS:
          return {32, 800};
       case Group_Params::HYBRID_SECP256R1_KYBER_768_R3_OQS:
          return {32, 1184};
+      case Group_Params::HYBRID_SECP256R1_eFRODOKEM_640_SHAKE_OQS:
+         return {32, 9616};
+      case Group_Params::HYBRID_SECP256R1_eFRODOKEM_640_AES_OQS:
+         return {32, 9616};
+
       case Group_Params::HYBRID_SECP384R1_KYBER_768_R3_OQS:
          return {48, 1184};
+      case Group_Params::HYBRID_SECP384R1_eFRODOKEM_976_SHAKE_OQS:
+         return {48, 15632};
+      case Group_Params::HYBRID_SECP384R1_eFRODOKEM_976_AES_OQS:
+         return {48, 15632};
+
+      case Group_Params::HYBRID_SECP521R1_eFRODOKEM_1344_SHAKE_OQS:
+         return {66, 21520};
+      case Group_Params::HYBRID_SECP521R1_eFRODOKEM_1344_AES_OQS:
+         return {66, 21520};
       case Group_Params::HYBRID_SECP521R1_KYBER_1024_R3_OQS:
          return {66, 1568};
 
@@ -202,6 +250,15 @@ std::vector<uint8_t> Hybrid_KEM_PublicKey::public_value() const {
    });
 }
 
+std::unique_ptr<Private_Key> Hybrid_KEM_PublicKey::generate_another(RandomNumberGenerator& rng) const {
+   std::vector<std::unique_ptr<Private_Key>> new_private_keys;
+   std::transform(
+      m_public_keys.begin(), m_public_keys.end(), std::back_inserter(new_private_keys), [&](const auto& public_key) {
+         return public_key->generate_another(rng);
+      });
+   return std::make_unique<Hybrid_KEM_PrivateKey>(std::move(new_private_keys));
+}
+
 bool Hybrid_KEM_PublicKey::supports_operation(PublicKeyOperation op) const {
    return PublicKeyOperation::KeyEncapsulation == op;
 }
